/***************************************************************************
 *   Copyright (C) 2010 - 2013 by Marcel Hasler                            *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This file is part of MPD Now Playing.                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "mpdhandler.h"

#include <KLocale>

MPDHandler::MPDHandler()
    : QObject(),
      m_connectionThread(0),
      m_idleThread(0),
      m_state(MPDStateUnknown),
      m_elapsedTime(0, 0, 0),
      m_totalTime(0, 0, 0)
{
    m_updateTimeTimer = new QTimer(this);
    m_updateTimeTimer->setSingleShot(false);
    m_updateTimeTimer->setInterval(1000);
    QObject::connect(m_updateTimeTimer, SIGNAL(timeout()), this, SLOT(slotTimeChanged()));
}


MPDHandler::~MPDHandler()
{
    disconnect();
    delete m_updateTimeTimer;
}


void MPDHandler::connect(const QString &host, unsigned int port, const QString &password)
{
    disconnect();

    m_host = host;
    m_port = port;
    m_password = password;

    m_connectionThread = new ConnectionThread(m_host, m_port, m_password);
    QObject::connect(m_connectionThread, SIGNAL(success(mpd_connection*)), this, SLOT(slotConnectionSuccess(mpd_connection*)));
    QObject::connect(m_connectionThread, SIGNAL(failed()), this, SLOT(slotConnectionFailed()));
    m_connectionThread->start();
}


void MPDHandler::disconnect()
{
    m_updateTimeTimer->stop();

    if (m_idleThread)
    {
        m_idleThread->stop();
        m_idleThread->wait();
        delete m_idleThread;
        m_idleThread = 0;
    }
}


void MPDHandler::play()
{
    mpd_connection *connection = connect();
    if (connection)
    {
        if (m_state == MPDStatePlay)
            mpd_run_pause(connection, true);
        else
            mpd_run_play(connection);

        mpd_connection_free(connection);
    }
}


void MPDHandler::stop()
{
    mpd_connection *connection = connect();
    if (connection)
    {
        mpd_run_stop(connection);
        mpd_connection_free(connection);
    }
}


void MPDHandler::next()
{
    mpd_connection *connection = connect();
    if (connection)
    {
        mpd_run_next(connection);
        mpd_connection_free(connection);
    }
}


void MPDHandler::prev()
{
    mpd_connection *connection = connect();
    if (connection)
    {
        mpd_run_previous(connection);
        mpd_connection_free(connection);
    }
}


MPDHandler::MPDState MPDHandler::state() const
{
    return m_state;
}


const QString& MPDHandler::artist() const
{
    return m_artist;
}


const QString& MPDHandler::title() const
{
    return m_title;
}


const QString& MPDHandler::album() const
{
    return m_album;
}


const QTime& MPDHandler::elapsedTime() const
{
    return m_elapsedTime;
}


const QTime& MPDHandler::totalTime() const
{
    return m_totalTime;
}


mpd_connection* MPDHandler::connect() const
{
    mpd_connection *connection = mpd_connection_new(m_host.toUtf8(), m_port, 1000);

    mpd_error error = mpd_connection_get_error(connection);
    if (error == MPD_ERROR_SUCCESS)
    {
        if (!m_password.isEmpty())
            mpd_run_password(connection, m_password.toUtf8());

        return connection;
    }

    return 0;
}


void MPDHandler::updateState()
{
    // Initialize to zero
    m_state = MPDStateUnknown;

    m_artist = QString::null;
    m_title = QString::null;
    m_album = QString::null;

    m_elapsedTime = QTime(0, 0, 0);
    m_totalTime = QTime(0, 0, 0);

    m_updateTimeTimer->stop();

    mpd_connection *connection = connect();
    if (!connection)
        return;

    // Update state and time
    mpd_status *status = mpd_run_status(connection);
    if (status)
    {
        mpd_state state = mpd_status_get_state(status);

        if (state == MPD_STATE_PLAY)
            m_state = MPDStatePlay;
        else if (state == MPD_STATE_PAUSE)
            m_state = MPDStatePause;
        else if (state == MPD_STATE_STOP)
            m_state = MPDStateStop;

        m_elapsedTime = m_elapsedTime.addSecs(mpd_status_get_elapsed_time(status));
        m_totalTime = m_totalTime.addSecs(mpd_status_get_total_time(status));

        if (m_state == MPDStatePlay)
            m_updateTimeTimer->start();

        mpd_status_free(status);
    }

    // Update track
    mpd_song *song = mpd_run_current_song(connection);
    if (song)
    {
        m_artist = QString::fromUtf8(mpd_song_get_tag(song, MPD_TAG_ARTIST, 0));
        m_title = QString::fromUtf8(mpd_song_get_tag(song, MPD_TAG_TITLE, 0));
        m_album = QString::fromUtf8(mpd_song_get_tag(song, MPD_TAG_ALBUM, 0));

        if (m_artist.isEmpty())
            m_artist = i18n("Unknown");
        if (m_title.isEmpty())
            m_title = i18n("Unknown");
        if (m_album.isEmpty())
            m_album = i18n("Unknown");

        mpd_song_free(song);
    }

    mpd_connection_free(connection);
}


void MPDHandler::updateTime()
{
    m_elapsedTime = m_elapsedTime.addSecs(1);

    // Synchronize with MPD every ten seconds
    int second = m_elapsedTime.second();
    if (second % 10 == 0)
    {
        mpd_connection *connection = connect();
        if (connection)
        {
            mpd_status *status = mpd_run_status(connection);
            if (status)
            {
                mpd_state state = mpd_status_get_state(status);
                if (state == MPD_STATE_PLAY || state == MPD_STATE_PAUSE)
                {
                    QTime elapsed(0, 0, 0);
                    m_elapsedTime = elapsed.addSecs(mpd_status_get_elapsed_time(status));
                }

                mpd_status_free(status);
            }

            mpd_connection_free(connection);
        }
    }
}


void MPDHandler::slotConnectionSuccess(mpd_connection *connection)
{
    m_connectionThread->wait();
    delete m_connectionThread;
    m_connectionThread = 0;

    m_idleThread = new IdleThread(connection);
    QObject::connect(m_idleThread, SIGNAL(stateChanged()), this, SLOT(slotStateChanged()));
    QObject::connect(m_idleThread, SIGNAL(connectionLost()), this, SLOT(slotConnectionLost()));
    m_idleThread->start();

    updateState();

    emit connectionSuccess();
}


void MPDHandler::slotConnectionFailed()
{
    m_connectionThread->wait();
    delete m_connectionThread;
    m_connectionThread = 0;

    emit connectionFailed();
}


void MPDHandler::slotConnectionLost()
{
    disconnect();
    emit connectionLost();
}


void MPDHandler::slotStateChanged()
{
    updateState();
    emit stateChanged();
}


void MPDHandler::slotTimeChanged()
{
    updateTime();
    emit timeChanged();
}
