/***************************************************************************
 *   Copyright (C) 2010 - 2013 by Marcel Hasler                            *
 *   mahasler@gmail.com                                                    *
 *                                                                         *
 *   This file is part of MPD Now Playing.                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program. If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#include "configurationwidget.h"
#include "mpdnowplaying.h"

#include <QGraphicsGridLayout>

#include <KConfigDialog>
#include <KLocale>

#include <Plasma/Theme>

#include <iostream>
using namespace std;

MPDNowPlaying::MPDNowPlaying(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    KGlobal::locale()->insertCatalog("mpdnowplaying");

    m_svg = new Plasma::Svg(this);
    m_svg->setImagePath("widgets/background");

    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    setBackgroundHints(DefaultBackground);
    resize(260, 150);
}


MPDNowPlaying::~MPDNowPlaying()
{
}


void MPDNowPlaying::init()
{
    readConfiguration();

    m_artistLabel = new Plasma::Label();
    m_artistText = new Plasma::Label();

    m_titleLabel = new Plasma::Label();
    m_titleText = new Plasma::Label();

    m_albumLabel = new Plasma::Label();
    m_albumText = new Plasma::Label();

    m_timeLabel = new Plasma::Label();
    m_timeText = new Plasma::Label();

    QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
    font.setBold(true);

    m_artistLabel->setFont(font);
    m_artistLabel->setText(i18n("Artist:"));
    m_artistLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 
    m_titleLabel->setFont(font);
    m_titleLabel->setText(i18n("Title:"));
    m_titleLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

    m_albumLabel->setFont(font);
    m_albumLabel->setText(i18n("Album:"));
    m_albumLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

    m_timeLabel->setFont(font);
    m_timeLabel->setText(i18n("Time:"));
    m_timeLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);

    font.setBold(false);

    m_artistText->setFont(font);
    m_artistText->setMinimumWidth(0);
    m_artistText->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_artistText->setWordWrap(false);
 
    m_titleText->setFont(font);
    m_titleText->setMinimumWidth(0);
    m_titleText->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_titleText->setWordWrap(false);

    m_albumText->setFont(font);
    m_albumText->setMinimumWidth(0);
    m_albumText->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_albumText->setWordWrap(false);

    m_timeText->setFont(font);
    m_timeText->setMinimumWidth(0);
    m_timeText->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    m_timeText->setWordWrap(false);

    QGraphicsGridLayout *infoLayout = new QGraphicsGridLayout();
    infoLayout->setColumnStretchFactor(0, 0);
    infoLayout->setColumnSpacing(0, 10);
    infoLayout->setColumnAlignment(0, Qt::AlignRight);

    infoLayout->addItem(m_artistLabel, 0, 0);
    infoLayout->addItem(m_artistText, 0, 1);

    infoLayout->addItem(m_titleLabel, 1, 0);
    infoLayout->addItem(m_titleText, 1, 1);

    infoLayout->addItem(m_albumLabel, 2, 0);
    infoLayout->addItem(m_albumText, 2, 1);

    infoLayout->addItem(m_timeLabel, 3, 0);
    infoLayout->addItem(m_timeText, 3, 1);

    m_playButton = new Plasma::IconWidget();
    m_stopButton = new Plasma::IconWidget();
    m_nextButton = new Plasma::IconWidget();
    m_prevButton = new Plasma::IconWidget();

    m_playButton->setIcon("media-playback-start");
    m_stopButton->setIcon("media-playback-stop");
    m_nextButton->setIcon("media-skip-forward");
    m_prevButton->setIcon("media-skip-backward");

    QGraphicsLinearLayout *controlsLayout = new QGraphicsLinearLayout(Qt::Horizontal);
    controlsLayout->addItem(m_prevButton);
    controlsLayout->addItem(m_playButton);
    controlsLayout->addItem(m_stopButton);
    controlsLayout->addItem(m_nextButton);

    m_layout = new QGraphicsLinearLayout(Qt::Vertical, this);
    m_layout->addItem(infoLayout);
    m_layout->addItem(controlsLayout);

    setLayout(m_layout);

    m_reconnectTimer = new QTimer(this);
    m_reconnectTimer->setSingleShot(true);
    m_reconnectTimer->setInterval(1000);
    QObject::connect(m_reconnectTimer, SIGNAL(timeout()), this, SLOT(connect()));

    m_mpdHandler = new MPDHandler();
    QObject::connect(m_mpdHandler, SIGNAL(connectionSuccess()), this, SLOT(enableUserInterface()));
    QObject::connect(m_mpdHandler, SIGNAL(connectionSuccess()), this, SLOT(updateState()));

    QObject::connect(m_mpdHandler, SIGNAL(connectionFailed()), this, SLOT(disableUserInterface()));
    QObject::connect(m_mpdHandler, SIGNAL(connectionFailed()), m_reconnectTimer, SLOT(start()));

    QObject::connect(m_mpdHandler, SIGNAL(connectionLost()), this, SLOT(disableUserInterface()));
    QObject::connect(m_mpdHandler, SIGNAL(connectionLost()), m_reconnectTimer, SLOT(start()));

    QObject::connect(m_mpdHandler, SIGNAL(stateChanged()), this, SLOT(updateState()));
    QObject::connect(m_mpdHandler, SIGNAL(timeChanged()), this, SLOT(updateTime()));

    QObject::connect(m_playButton, SIGNAL(clicked()), m_mpdHandler, SLOT(play()));
    QObject::connect(m_stopButton, SIGNAL(clicked()), m_mpdHandler, SLOT(stop()));
    QObject::connect(m_nextButton, SIGNAL(clicked()), m_mpdHandler, SLOT(next()));
    QObject::connect(m_prevButton, SIGNAL(clicked()), m_mpdHandler, SLOT(prev()));

    disableUserInterface();
    connect();
}


void MPDNowPlaying::connect()
{
    m_mpdHandler->connect(m_settings.host, m_settings.port, m_settings.password);
}


void MPDNowPlaying::setUserInterfaceEnabled(bool enable)
{
    m_artistLabel->nativeWidget()->setEnabled(enable);
    m_titleLabel->nativeWidget()->setEnabled(enable);
    m_albumLabel->nativeWidget()->setEnabled(enable);
    m_timeLabel->nativeWidget()->setEnabled(enable);

    m_playButton->setEnabled(enable);
    m_stopButton->setEnabled(enable);
    m_prevButton->setEnabled(enable);
    m_nextButton->setEnabled(enable);
}


void MPDNowPlaying::enableUserInterface()
{
    setUserInterfaceEnabled(true);
}


void MPDNowPlaying::disableUserInterface()
{
    clearText();
    setUserInterfaceEnabled(false);
}


void MPDNowPlaying::updateState()
{
    MPDHandler::MPDState state = m_mpdHandler->state();

    if (state == MPDHandler::MPDStatePlay || state == MPDHandler::MPDStatePause)
    {
        updateText();
        updateTime();
    }
    else
        clearText();

    if (state == MPDHandler::MPDStatePlay)
        m_playButton->setIcon("media-playback-pause");
    else
        m_playButton->setIcon("media-playback-start");
}


void MPDNowPlaying::updateText()
{
    m_artistText->setText(m_mpdHandler->artist());
    m_titleText->setText(m_mpdHandler->title());
    m_albumText->setText(m_mpdHandler->album());
}


void MPDNowPlaying::updateTime()
{
    m_timeText->setText(m_mpdHandler->elapsedTime().toString("m:ss") + " / " +
                        m_mpdHandler->totalTime().toString("m:ss"));
}


void MPDNowPlaying::clearText()
{
    m_artistText->setText("");
    m_titleText->setText("");
    m_albumText->setText("");
    m_timeText->setText("");
}


void MPDNowPlaying::createConfigurationInterface(KConfigDialog *parent)
{
    ConfigurationWidget *widget = new ConfigurationWidget(parent, &m_settings);
    parent->addPage(widget, i18n("Connection"), icon());
    QObject::connect(widget, SIGNAL(settingsChanged()), this, SLOT(saveConfiguration()));
    QObject::connect(widget, SIGNAL(settingsChanged()), this, SLOT(connect()));
}


void MPDNowPlaying::saveConfiguration()
{
    KConfigGroup group = config("Connection");
    group.writeEntry("UseDefault", m_settings.useDefault);
    group.writeEntry("Host", m_settings.host);
    group.writeEntry("Port", m_settings.port);
    group.writeEntry("Password", m_settings.password);

    emit configNeedsSaving();
}


void MPDNowPlaying::readConfiguration()
{
    KConfigGroup group = config("Connection");
    m_settings.useDefault = group.readEntry("UseDefault", true);
    if (m_settings.useDefault)
    {
        m_settings.host = "localhost";
        m_settings.port = 6600;
        m_settings.password = QString();
    }
    else
    {
        m_settings.host = group.readEntry("Host", "localhost");
        m_settings.port = group.readEntry("Port", 6600);
        m_settings.password = group.readEntry("Password", QString());
    }
}

K_EXPORT_PLASMA_APPLET(mpdnowplaying, MPDNowPlaying)
